"""
CSSL GUI Builder - Direct module execution.
Usage: python -m includecpp.gui.guimaker [optional_file.cssl]
"""

import sys
from . import run_guimaker

file_path = sys.argv[1] if len(sys.argv) > 1 else ""
run_guimaker(file_path)



"""
CSSL GUI Builder - QGraphicsItem subclasses for widget visual representations.
Each widget type gets a custom painter that draws a realistic preview.
"""

from PyQt6.QtWidgets import QGraphicsRectItem, QGraphicsItem, QGraphicsSceneMouseEvent
from PyQt6.QtCore import Qt, QRectF, QPointF
from PyQt6.QtGui import QPainter, QColor, QFont, QPen, QBrush, QFontMetrics, QPainterPath

from .models import WidgetModel
from .constants import CsslWidgetType, WIDGET_REGISTRY
from . import theme


# ---------------------------------------------------------------------------
# Resize handles
# ---------------------------------------------------------------------------

class ResizeHandle(QGraphicsRectItem):
    """Small square handle for resizing widgets."""

    HANDLE_SIZE = 7

    # Handle positions
    TOP_LEFT = 0
    TOP_CENTER = 1
    TOP_RIGHT = 2
    MIDDLE_LEFT = 3
    MIDDLE_RIGHT = 4
    BOTTOM_LEFT = 5
    BOTTOM_CENTER = 6
    BOTTOM_RIGHT = 7

    def __init__(self, position: int, parent_item: 'CanvasWidgetItem'):
        super().__init__(-self.HANDLE_SIZE / 2, -self.HANDLE_SIZE / 2,
                         self.HANDLE_SIZE, self.HANDLE_SIZE, parent_item)
        self._position = position
        self._parent_item = parent_item
        self.setBrush(QBrush(QColor(theme.HANDLE_COLOR)))
        self.setPen(QPen(QColor(theme.DARK_BG), 1))
        self.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsMovable, False)
        self.setZValue(1000)
        self.setCursor(self._get_cursor())
        self.setVisible(False)

    def _get_cursor(self) -> Qt.CursorShape:
        cursors = {
            self.TOP_LEFT: Qt.CursorShape.SizeFDiagCursor,
            self.TOP_RIGHT: Qt.CursorShape.SizeBDiagCursor,
            self.BOTTOM_LEFT: Qt.CursorShape.SizeBDiagCursor,
            self.BOTTOM_RIGHT: Qt.CursorShape.SizeFDiagCursor,
            self.TOP_CENTER: Qt.CursorShape.SizeVerCursor,
            self.BOTTOM_CENTER: Qt.CursorShape.SizeVerCursor,
            self.MIDDLE_LEFT: Qt.CursorShape.SizeHorCursor,
            self.MIDDLE_RIGHT: Qt.CursorShape.SizeHorCursor,
        }
        return cursors.get(self._position, Qt.CursorShape.ArrowCursor)

    def update_position(self, rect: QRectF) -> None:
        """Update handle position based on parent rect."""
        positions = {
            self.TOP_LEFT: rect.topLeft(),
            self.TOP_CENTER: QPointF(rect.center().x(), rect.top()),
            self.TOP_RIGHT: rect.topRight(),
            self.MIDDLE_LEFT: QPointF(rect.left(), rect.center().y()),
            self.MIDDLE_RIGHT: QPointF(rect.right(), rect.center().y()),
            self.BOTTOM_LEFT: rect.bottomLeft(),
            self.BOTTOM_CENTER: QPointF(rect.center().x(), rect.bottom()),
            self.BOTTOM_RIGHT: rect.bottomRight(),
        }
        pos = positions.get(self._position, QPointF(0, 0))
        self.setPos(pos)


# ---------------------------------------------------------------------------
# Base canvas widget item
# ---------------------------------------------------------------------------

class CanvasWidgetItem(QGraphicsRectItem):
    """Base class for all widget representations on the canvas."""

    MIN_WIDTH = 20
    MIN_HEIGHT = 15

    def __init__(self, widget_model: WidgetModel, parent_item: QGraphicsItem = None):
        super().__init__(0, 0, widget_model.width, widget_model.height, parent_item)
        self._model = widget_model
        self._selected = False
        self._dragging = False
        self._resizing = False
        self._resize_handle_pos = -1
        self._drag_start_pos = QPointF()
        self._drag_start_scene_pos = QPointF()
        self._resize_start_rect = QRectF()
        self._resize_start_mouse = QPointF()
        self._handles: list[ResizeHandle] = []

        self.setPos(widget_model.x, widget_model.y)
        self.setFlag(QGraphicsItem.GraphicsItemFlag.ItemIsSelectable, True)
        self.setFlag(QGraphicsItem.GraphicsItemFlag.ItemSendsGeometryChanges, True)
        self.setAcceptHoverEvents(True)

        self._create_handles()

    @property
    def model(self) -> WidgetModel:
        return self._model

    def _create_handles(self) -> None:
        for i in range(8):
            handle = ResizeHandle(i, self)
            self._handles.append(handle)

    def set_selected(self, selected: bool) -> None:
        self._selected = selected
        for handle in self._handles:
            handle.setVisible(selected)
        if selected:
            self._update_handle_positions()
        self.update()

    def _update_handle_positions(self) -> None:
        rect = self.rect()
        for handle in self._handles:
            handle.update_position(rect)

    def update_from_model(self) -> None:
        """Sync visual representation with model data."""
        self.setPos(self._model.x, self._model.y)
        self.setRect(0, 0, self._model.width, self._model.height)
        self._update_handle_positions()
        self.update()

    def paint(self, painter: QPainter, option, widget=None) -> None:
        """Default widget painting - subclasses override for specific looks."""
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()

        # Background
        painter.setBrush(QBrush(QColor(theme.WIDGET_BG)))
        painter.setPen(QPen(QColor(theme.WIDGET_BORDER), 1))
        painter.drawRoundedRect(rect, 3, 3)

        # Type label centered
        painter.setPen(QPen(QColor(theme.WIDGET_TEXT)))
        font = QFont("Segoe UI", 10)
        painter.setFont(font)
        name = self._model.name or self._model.widget_type.value
        painter.drawText(rect, Qt.AlignmentFlag.AlignCenter, name)

        # Selection highlight
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2, Qt.PenStyle.DashLine))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))

    def mousePressEvent(self, event: QGraphicsSceneMouseEvent) -> None:
        if event.button() == Qt.MouseButton.LeftButton:
            # Check if we're on a resize handle
            handle_pos = self._hit_test_handle(event.pos())
            if handle_pos >= 0:
                self._resizing = True
                self._resize_handle_pos = handle_pos
                self._resize_start_rect = QRectF(
                    self._model.x, self._model.y,
                    self._model.width, self._model.height
                )
                self._resize_start_mouse = event.scenePos()
                event.accept()
                return

            self._dragging = True
            self._drag_start_pos = QPointF(self._model.x, self._model.y)
            self._drag_start_scene_pos = event.scenePos()
        super().mousePressEvent(event)

    def mouseMoveEvent(self, event: QGraphicsSceneMouseEvent) -> None:
        if self._resizing:
            self._do_resize(event.scenePos())
            event.accept()
            return

        if self._dragging:
            delta = event.scenePos() - self._drag_start_scene_pos
            new_x = int(self._drag_start_pos.x() + delta.x())
            new_y = int(self._drag_start_pos.y() + delta.y())
            # Snap to grid
            grid = 10
            new_x = round(new_x / grid) * grid
            new_y = round(new_y / grid) * grid
            self.setPos(new_x, new_y)
            event.accept()
            return
        super().mouseMoveEvent(event)

    def mouseReleaseEvent(self, event: QGraphicsSceneMouseEvent) -> None:
        if self._resizing:
            self._resizing = False
            self._resize_handle_pos = -1
            # Notify canvas of resize completion
            canvas = self.scene()
            if canvas and hasattr(canvas, '_on_item_resized'):
                canvas._on_item_resized(self)
            event.accept()
            return

        if self._dragging:
            self._dragging = False
            new_pos = self.pos()
            new_x = int(new_pos.x())
            new_y = int(new_pos.y())
            if new_x != int(self._drag_start_pos.x()) or \
               new_y != int(self._drag_start_pos.y()):
                canvas = self.scene()
                if canvas and hasattr(canvas, '_on_item_moved'):
                    canvas._on_item_moved(
                        self,
                        int(self._drag_start_pos.x()),
                        int(self._drag_start_pos.y()),
                        new_x, new_y
                    )
            event.accept()
            return
        super().mouseReleaseEvent(event)

    def _hit_test_handle(self, pos: QPointF) -> int:
        """Check if position hits any resize handle. Returns handle index or -1."""
        if not self._selected:
            return -1
        rect = self.rect()
        margin = 8
        corners = [
            (rect.topLeft(), ResizeHandle.TOP_LEFT),
            (QPointF(rect.center().x(), rect.top()), ResizeHandle.TOP_CENTER),
            (rect.topRight(), ResizeHandle.TOP_RIGHT),
            (QPointF(rect.left(), rect.center().y()), ResizeHandle.MIDDLE_LEFT),
            (QPointF(rect.right(), rect.center().y()), ResizeHandle.MIDDLE_RIGHT),
            (rect.bottomLeft(), ResizeHandle.BOTTOM_LEFT),
            (QPointF(rect.center().x(), rect.bottom()), ResizeHandle.BOTTOM_CENTER),
            (rect.bottomRight(), ResizeHandle.BOTTOM_RIGHT),
        ]
        for corner_pos, handle_idx in corners:
            if abs(pos.x() - corner_pos.x()) < margin and \
               abs(pos.y() - corner_pos.y()) < margin:
                return handle_idx
        return -1

    def _do_resize(self, scene_pos: QPointF) -> None:
        """Handle resize drag."""
        delta = scene_pos - self._resize_start_mouse
        r = self._resize_start_rect
        hp = self._resize_handle_pos
        new_x, new_y = r.x(), r.y()
        new_w, new_h = r.width(), r.height()

        if hp in (ResizeHandle.TOP_LEFT, ResizeHandle.MIDDLE_LEFT, ResizeHandle.BOTTOM_LEFT):
            new_x = r.x() + delta.x()
            new_w = r.width() - delta.x()
        if hp in (ResizeHandle.TOP_RIGHT, ResizeHandle.MIDDLE_RIGHT, ResizeHandle.BOTTOM_RIGHT):
            new_w = r.width() + delta.x()
        if hp in (ResizeHandle.TOP_LEFT, ResizeHandle.TOP_CENTER, ResizeHandle.TOP_RIGHT):
            new_y = r.y() + delta.y()
            new_h = r.height() - delta.y()
        if hp in (ResizeHandle.BOTTOM_LEFT, ResizeHandle.BOTTOM_CENTER, ResizeHandle.BOTTOM_RIGHT):
            new_h = r.height() + delta.y()

        # Enforce minimum size
        if new_w < self.MIN_WIDTH:
            if hp in (ResizeHandle.TOP_LEFT, ResizeHandle.MIDDLE_LEFT, ResizeHandle.BOTTOM_LEFT):
                new_x = r.x() + r.width() - self.MIN_WIDTH
            new_w = self.MIN_WIDTH
        if new_h < self.MIN_HEIGHT:
            if hp in (ResizeHandle.TOP_LEFT, ResizeHandle.TOP_CENTER, ResizeHandle.TOP_RIGHT):
                new_y = r.y() + r.height() - self.MIN_HEIGHT
            new_h = self.MIN_HEIGHT

        # Snap to grid
        grid = 10
        new_x = round(new_x / grid) * grid
        new_y = round(new_y / grid) * grid
        new_w = max(self.MIN_WIDTH, round(new_w / grid) * grid)
        new_h = max(self.MIN_HEIGHT, round(new_h / grid) * grid)

        self._model.x = int(new_x)
        self._model.y = int(new_y)
        self._model.width = int(new_w)
        self._model.height = int(new_h)
        self.setPos(new_x, new_y)
        self.setRect(0, 0, new_w, new_h)
        self._update_handle_positions()


# ---------------------------------------------------------------------------
# Widget-specific painters
# ---------------------------------------------------------------------------

class ButtonItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        bg = self._model.get_property("bg_color", "")
        fg = self._model.get_property("fg_color", "")
        font_size = self._model.get_property("font_size", 0)
        enabled = self._model.get_property("enabled", True)
        # Button body
        bg_color = QColor(bg) if bg else QColor("#e0e0e0")
        if not enabled:
            bg_color.setAlpha(120)
        painter.setBrush(QBrush(bg_color))
        painter.setPen(QPen(QColor("#aaaaaa"), 1))
        painter.drawRoundedRect(rect.adjusted(1, 1, -1, -1), 4, 4)
        # Text
        text = self._model.get_property("text", "Button")
        fg_color = QColor(fg) if fg else QColor("#333333")
        if not enabled:
            fg_color.setAlpha(120)
        painter.setPen(QPen(fg_color))
        painter.setFont(QFont("Segoe UI", font_size if font_size > 0 else 10))
        painter.drawText(rect, Qt.AlignmentFlag.AlignCenter, str(text))
        # Selection
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRoundedRect(rect.adjusted(-1, -1, 1, 1), 4, 4)


class LabelItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        bg = self._model.get_property("bg_color", "")
        fg = self._model.get_property("fg_color", "")
        alignment = self._model.get_property("alignment", "left")
        bold = self._model.get_property("bold", False)
        font_size = self._model.get_property("font_size", 0)
        font_family = self._model.get_property("font_family", "")
        # Background
        if bg:
            painter.setBrush(QBrush(QColor(bg)))
            painter.setPen(QPen(QColor("#cccccc"), 1))
        else:
            painter.setBrush(QBrush(QColor(255, 255, 255, 30)))
            painter.setPen(QPen(QColor("#cccccc"), 1, Qt.PenStyle.DotLine))
        painter.drawRect(rect)
        # Text
        text = self._model.get_property("text", "Label")
        painter.setPen(QPen(QColor(fg) if fg else QColor("#333333")))
        font = QFont(font_family if font_family else "Segoe UI",
                     font_size if font_size > 0 else 10)
        if bold:
            font.setBold(True)
        painter.setFont(font)
        align_map = {"left": Qt.AlignmentFlag.AlignLeft,
                     "center": Qt.AlignmentFlag.AlignHCenter,
                     "right": Qt.AlignmentFlag.AlignRight}
        h_align = align_map.get(alignment, Qt.AlignmentFlag.AlignLeft)
        painter.drawText(rect.adjusted(4, 0, -4, 0),
                         h_align | Qt.AlignmentFlag.AlignVCenter,
                         str(text))
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class InputFieldItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        bg = self._model.get_property("bg_color", "")
        fg = self._model.get_property("fg_color", "")
        # Input field look
        painter.setBrush(QBrush(QColor(bg) if bg else QColor("#ffffff")))
        painter.setPen(QPen(QColor("#888888"), 1))
        painter.drawRoundedRect(rect.adjusted(1, 1, -1, -1), 3, 3)
        # Placeholder/text
        text = self._model.get_property("text", "")
        placeholder = self._model.get_property("placeholder", "")
        display = text if text else placeholder
        if text:
            text_color = QColor(fg) if fg else QColor("#333333")
        else:
            text_color = QColor("#aaaaaa")
        painter.setPen(QPen(text_color))
        painter.setFont(QFont("Segoe UI", 10))
        painter.drawText(rect.adjusted(8, 0, -8, 0),
                         Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter,
                         str(display) if display else "")
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class TextAreaItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        bg = self._model.get_property("bg_color", "")
        fg = self._model.get_property("fg_color", "")
        font_family = self._model.get_property("font_family", "")
        font_size = self._model.get_property("font_size", 0)
        painter.setBrush(QBrush(QColor(bg) if bg else QColor("#ffffff")))
        painter.setPen(QPen(QColor("#888888"), 1))
        painter.drawRoundedRect(rect.adjusted(1, 1, -1, -1), 3, 3)
        # Draw lines to suggest multiline
        painter.setPen(QPen(QColor("#e0e0e0"), 1))
        y = rect.top() + 24
        while y < rect.bottom() - 8:
            painter.drawLine(int(rect.left() + 8), int(y),
                             int(rect.right() - 8), int(y))
            y += 18
        # Text content
        font = QFont(font_family if font_family else "Segoe UI",
                     font_size if font_size > 0 else 9)
        painter.setFont(font)
        text = self._model.get_property("text", "")
        if text:
            painter.setPen(QPen(QColor(fg) if fg else QColor("#333333")))
            painter.drawText(rect.adjusted(8, 6, -8, -6),
                             Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignTop,
                             str(text)[:100])
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class CheckBoxItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        # Background
        painter.setBrush(QBrush(QColor(255, 255, 255, 30)))
        painter.setPen(QPen(QColor("#cccccc"), 1, Qt.PenStyle.DotLine))
        painter.drawRect(rect)
        # Checkbox square
        box_size = 16
        box_y = int(rect.center().y() - box_size / 2)
        box_rect = QRectF(rect.left() + 4, box_y, box_size, box_size)
        checked = self._model.get_property("checked", False)
        painter.setBrush(QBrush(QColor("#3b82f6") if checked else QColor("#ffffff")))
        painter.setPen(QPen(QColor("#888888"), 1))
        painter.drawRoundedRect(box_rect, 3, 3)
        if checked:
            painter.setPen(QPen(QColor("#ffffff"), 2))
            painter.drawLine(int(box_rect.left() + 3), int(box_rect.center().y()),
                             int(box_rect.center().x()), int(box_rect.bottom() - 3))
            painter.drawLine(int(box_rect.center().x()), int(box_rect.bottom() - 3),
                             int(box_rect.right() - 3), int(box_rect.top() + 3))
        # Text
        text = self._model.get_property("text", "Checkbox")
        painter.setPen(QPen(QColor("#333333")))
        painter.setFont(QFont("Segoe UI", 10))
        text_rect = QRectF(rect.left() + box_size + 10, rect.top(),
                           rect.width() - box_size - 14, rect.height())
        painter.drawText(text_rect, Qt.AlignmentFlag.AlignLeft |
                         Qt.AlignmentFlag.AlignVCenter, str(text))
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class RadioButtonItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        painter.setBrush(QBrush(QColor(255, 255, 255, 30)))
        painter.setPen(QPen(QColor("#cccccc"), 1, Qt.PenStyle.DotLine))
        painter.drawRect(rect)
        # Radio circle
        circle_r = 8
        cx = rect.left() + 4 + circle_r
        cy = rect.center().y()
        painter.setBrush(QBrush(QColor("#ffffff")))
        painter.setPen(QPen(QColor("#888888"), 1))
        painter.drawEllipse(QPointF(cx, cy), circle_r, circle_r)
        # Inner dot (selected state placeholder)
        painter.setBrush(QBrush(QColor("#888888")))
        painter.setPen(Qt.PenStyle.NoPen)
        painter.drawEllipse(QPointF(cx, cy), 4, 4)
        # Text
        text = self._model.get_property("text", "Option")
        painter.setPen(QPen(QColor("#333333")))
        painter.setFont(QFont("Segoe UI", 10))
        text_rect = QRectF(rect.left() + circle_r * 2 + 12, rect.top(),
                           rect.width() - circle_r * 2 - 16, rect.height())
        painter.drawText(text_rect, Qt.AlignmentFlag.AlignLeft |
                         Qt.AlignmentFlag.AlignVCenter, str(text))
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class ComboBoxItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        painter.setBrush(QBrush(QColor("#ffffff")))
        painter.setPen(QPen(QColor("#888888"), 1))
        painter.drawRoundedRect(rect.adjusted(1, 1, -1, -1), 3, 3)
        # Down arrow
        arrow_x = rect.right() - 20
        arrow_y = rect.center().y()
        painter.setPen(QPen(QColor("#666666"), 2))
        painter.drawLine(int(arrow_x - 4), int(arrow_y - 2),
                         int(arrow_x), int(arrow_y + 2))
        painter.drawLine(int(arrow_x), int(arrow_y + 2),
                         int(arrow_x + 4), int(arrow_y - 2))
        # Items preview
        items = self._model.get_property("items", [])
        display = items[0] if items else "Select..."
        painter.setPen(QPen(QColor("#333333" if items else "#aaaaaa")))
        painter.setFont(QFont("Segoe UI", 10))
        painter.drawText(rect.adjusted(8, 0, -28, 0),
                         Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter,
                         str(display))
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class SliderItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        painter.setBrush(QBrush(QColor(255, 255, 255, 30)))
        painter.setPen(QPen(QColor("#cccccc"), 1, Qt.PenStyle.DotLine))
        painter.drawRect(rect)
        # Track
        track_y = rect.center().y()
        painter.setPen(Qt.PenStyle.NoPen)
        painter.setBrush(QBrush(QColor("#cccccc")))
        track_rect = QRectF(rect.left() + 8, track_y - 2, rect.width() - 16, 4)
        painter.drawRoundedRect(track_rect, 2, 2)
        # Thumb
        min_v = self._model.get_property("min_val", 0)
        max_v = self._model.get_property("max_val", 100)
        val = self._model.get_property("value", 0)
        ratio = (val - min_v) / max(1, max_v - min_v)
        thumb_x = rect.left() + 8 + ratio * (rect.width() - 16)
        painter.setBrush(QBrush(QColor("#3b82f6")))
        painter.drawEllipse(QPointF(thumb_x, track_y), 7, 7)
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class ProgressBarItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        # Background
        painter.setBrush(QBrush(QColor("#e0e0e0")))
        painter.setPen(QPen(QColor("#cccccc"), 1))
        painter.drawRoundedRect(rect.adjusted(1, 1, -1, -1), 3, 3)
        # Progress fill
        val = self._model.get_property("value", 0)
        max_v = self._model.get_property("max_val", 100)
        ratio = val / max(1, max_v)
        fill_w = (rect.width() - 4) * ratio
        if fill_w > 0:
            fill_rect = QRectF(rect.left() + 2, rect.top() + 2, fill_w, rect.height() - 4)
            painter.setBrush(QBrush(QColor("#3b82f6")))
            painter.setPen(Qt.PenStyle.NoPen)
            painter.drawRoundedRect(fill_rect, 2, 2)
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class ListBoxItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        painter.setBrush(QBrush(QColor("#ffffff")))
        painter.setPen(QPen(QColor("#888888"), 1))
        painter.drawRoundedRect(rect.adjusted(1, 1, -1, -1), 3, 3)
        # Draw sample items
        items = self._model.get_property("items", [])
        painter.setFont(QFont("Segoe UI", 10))
        y = rect.top() + 4
        for i, item in enumerate(items[:8]):
            if y + 22 > rect.bottom():
                break
            item_rect = QRectF(rect.left() + 4, y, rect.width() - 8, 20)
            if i == 0:
                painter.setBrush(QBrush(QColor("#e8f0fe")))
                painter.setPen(Qt.PenStyle.NoPen)
                painter.drawRoundedRect(item_rect, 2, 2)
            painter.setPen(QPen(QColor("#333333")))
            painter.drawText(item_rect.adjusted(6, 0, -6, 0),
                             Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter,
                             str(item))
            y += 22
        if not items:
            painter.setPen(QPen(QColor("#aaaaaa")))
            painter.drawText(rect, Qt.AlignmentFlag.AlignCenter, "List Box")
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class TableItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        painter.setBrush(QBrush(QColor("#ffffff")))
        painter.setPen(QPen(QColor("#888888"), 1))
        painter.drawRect(rect)
        # Header
        columns = self._model.get_property("columns", [])
        header_h = 28
        header_rect = QRectF(rect.left(), rect.top(), rect.width(), header_h)
        painter.setBrush(QBrush(QColor("#f0f0f0")))
        painter.drawRect(header_rect)
        if columns:
            col_w = rect.width() / len(columns)
            painter.setFont(QFont("Segoe UI", 9, QFont.Weight.Bold))
            painter.setPen(QPen(QColor("#333333")))
            for i, col in enumerate(columns):
                cr = QRectF(rect.left() + i * col_w, rect.top(), col_w, header_h)
                painter.drawText(cr, Qt.AlignmentFlag.AlignCenter, str(col))
            # Column lines
            painter.setPen(QPen(QColor("#dddddd"), 1))
            for i in range(1, len(columns)):
                x = rect.left() + i * col_w
                painter.drawLine(int(x), int(rect.top()), int(x), int(rect.bottom()))
        else:
            painter.setPen(QPen(QColor("#aaaaaa")))
            painter.setFont(QFont("Segoe UI", 10))
            painter.drawText(rect, Qt.AlignmentFlag.AlignCenter, "Table")
        # Row lines
        painter.setPen(QPen(QColor("#eeeeee"), 1))
        y = rect.top() + header_h
        while y < rect.bottom():
            painter.drawLine(int(rect.left()), int(y), int(rect.right()), int(y))
            y += 24
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class SeparatorItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        rect = self.rect()
        orientation = self._model.get_property("orientation", "horizontal")
        color = self._model.get_property("color", "")
        thickness = self._model.get_property("thickness", 2)
        painter.setPen(QPen(QColor(color) if color else QColor("#cccccc"), thickness))
        if orientation == "horizontal":
            y = rect.center().y()
            painter.drawLine(int(rect.left()), int(y), int(rect.right()), int(y))
        else:
            x = rect.center().x()
            painter.drawLine(int(x), int(rect.top()), int(x), int(rect.bottom()))
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2, Qt.PenStyle.DashLine))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class PictureItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        painter.setBrush(QBrush(QColor("#f5f5f5")))
        painter.setPen(QPen(QColor("#cccccc"), 1))
        painter.drawRect(rect)
        # Image icon placeholder
        painter.setPen(QPen(QColor("#aaaaaa")))
        painter.setFont(QFont("Segoe UI", 24))
        painter.drawText(rect, Qt.AlignmentFlag.AlignCenter, "\u25a6")
        # Path label
        path = self._model.get_property("image_path", "")
        if path:
            painter.setFont(QFont("Segoe UI", 8))
            painter.setPen(QPen(QColor("#888888")))
            painter.drawText(rect.adjusted(4, 0, -4, -4),
                             Qt.AlignmentFlag.AlignBottom | Qt.AlignmentFlag.AlignCenter,
                             str(path)[-30:])
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class TimePickerItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        painter.setBrush(QBrush(QColor("#ffffff")))
        painter.setPen(QPen(QColor("#888888"), 1))
        painter.drawRoundedRect(rect.adjusted(1, 1, -1, -1), 3, 3)
        h = self._model.get_property("hour", 0)
        m = self._model.get_property("minute", 0)
        s = self._model.get_property("second", 0)
        painter.setPen(QPen(QColor("#333333")))
        painter.setFont(QFont("Segoe UI", 11))
        painter.drawText(rect, Qt.AlignmentFlag.AlignCenter, f"{h:02d}:{m:02d}:{s:02d}")
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class DatePickerItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        painter.setBrush(QBrush(QColor("#ffffff")))
        painter.setPen(QPen(QColor("#888888"), 1))
        painter.drawRoundedRect(rect.adjusted(1, 1, -1, -1), 3, 3)
        y = self._model.get_property("year", 2025)
        m = self._model.get_property("month", 1)
        d = self._model.get_property("day", 1)
        painter.setPen(QPen(QColor("#333333")))
        painter.setFont(QFont("Segoe UI", 11))
        painter.drawText(rect, Qt.AlignmentFlag.AlignCenter, f"{y:04d}-{m:02d}-{d:02d}")
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class TabControlItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        # Tab bar area
        tab_h = 30
        painter.setBrush(QBrush(QColor("#f0f0f0")))
        painter.setPen(QPen(QColor("#cccccc"), 1))
        painter.drawRect(QRectF(rect.left(), rect.top(), rect.width(), tab_h))
        # Sample tabs
        tabs = ["Tab 1", "Tab 2", "Tab 3"]
        x = rect.left() + 2
        painter.setFont(QFont("Segoe UI", 9))
        for i, tab in enumerate(tabs):
            tw = 70
            tr = QRectF(x, rect.top() + 2, tw, tab_h - 4)
            if i == 0:
                painter.setBrush(QBrush(QColor("#ffffff")))
                painter.setPen(QPen(QColor("#3b82f6"), 0))
                painter.drawRoundedRect(tr, 3, 3)
                painter.setPen(QPen(QColor("#333333")))
            else:
                painter.setPen(QPen(QColor("#888888")))
            painter.drawText(tr, Qt.AlignmentFlag.AlignCenter, tab)
            x += tw + 4
        # Content area
        content_rect = QRectF(rect.left(), rect.top() + tab_h, rect.width(),
                              rect.height() - tab_h)
        painter.setBrush(QBrush(QColor("#ffffff")))
        painter.setPen(QPen(QColor("#cccccc"), 1))
        painter.drawRect(content_rect)
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class ScrollPanelItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        painter.setRenderHint(QPainter.RenderHint.Antialiasing)
        rect = self.rect()
        painter.setBrush(QBrush(QColor("#ffffff")))
        painter.setPen(QPen(QColor("#888888"), 1))
        painter.drawRect(rect)
        # Scrollbar indicator
        sb_w = 10
        sb_rect = QRectF(rect.right() - sb_w - 2, rect.top() + 4,
                         sb_w, rect.height() * 0.4)
        painter.setBrush(QBrush(QColor("#cccccc")))
        painter.setPen(Qt.PenStyle.NoPen)
        painter.drawRoundedRect(sb_rect, 4, 4)
        # Label
        painter.setPen(QPen(QColor("#aaaaaa")))
        painter.setFont(QFont("Segoe UI", 9))
        painter.drawText(rect, Qt.AlignmentFlag.AlignCenter, "Scroll Panel")
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class MenuBarItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        rect = self.rect()
        painter.setBrush(QBrush(QColor("#f0f0f0")))
        painter.setPen(QPen(QColor("#cccccc"), 1))
        painter.drawRect(rect)
        # Draw menus from property or fallback
        menus_data = self._model.get_property("menus", [])
        if menus_data:
            menu_labels = [m.get("label", "Menu") for m in menus_data
                           if isinstance(m, dict)]
        else:
            menu_labels = ["File", "Edit", "View", "Help"]
        painter.setPen(QPen(QColor("#333333")))
        painter.setFont(QFont("Segoe UI", 10))
        x = rect.left() + 8
        for label in menu_labels:
            fm = QFontMetrics(painter.font())
            tw = fm.horizontalAdvance(label) + 16
            painter.drawText(QRectF(x, rect.top(), tw, rect.height()),
                             Qt.AlignmentFlag.AlignCenter, label)
            x += tw
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


class ToolbarItem(CanvasWidgetItem):
    def paint(self, painter: QPainter, option, widget=None) -> None:
        rect = self.rect()
        painter.setBrush(QBrush(QColor("#f5f5f5")))
        painter.setPen(QPen(QColor("#cccccc"), 1))
        painter.drawRect(rect)
        # Draw toolbar slots from property or fallback
        slots = self._model.get_property("slots", [])
        x = rect.left() + 6
        y = rect.center().y()
        btn_h = 28
        slot_font = QFont("Segoe UI", 8)
        fm = QFontMetrics(slot_font)
        if slots:
            for slot in slots:
                if not isinstance(slot, dict):
                    continue
                tooltip = slot.get("tooltip", "")
                # Auto-size width: measure text + padding, minimum 28px
                text_w = fm.horizontalAdvance(tooltip) if tooltip else 0
                btn_w = max(btn_h, text_w + 16)
                br = QRectF(x, y - btn_h / 2, btn_w, btn_h)
                painter.setBrush(QBrush(QColor("#e0e0e0")))
                painter.setPen(QPen(QColor("#bbbbbb"), 1))
                painter.drawRoundedRect(br, 3, 3)
                if tooltip:
                    painter.setPen(QPen(QColor("#555555")))
                    painter.setFont(slot_font)
                    painter.drawText(br, Qt.AlignmentFlag.AlignCenter, tooltip)
                x += btn_w + 4
        else:
            # Fallback: 5 generic placeholder buttons
            for i in range(5):
                br = QRectF(x, y - btn_h / 2, btn_h, btn_h)
                painter.setBrush(QBrush(QColor("#e0e0e0")))
                painter.setPen(QPen(QColor("#bbbbbb"), 1))
                painter.drawRoundedRect(br, 3, 3)
                x += btn_h + 4
            painter.setPen(QPen(QColor("#888888")))
            painter.setFont(QFont("Segoe UI", 9))
            painter.drawText(QRectF(x + 8, rect.top(), 80, rect.height()),
                             Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter,
                             "Toolbar")
        if self._selected:
            painter.setBrush(Qt.BrushStyle.NoBrush)
            painter.setPen(QPen(QColor(theme.SELECTION_COLOR), 2))
            painter.drawRect(rect.adjusted(-1, -1, 1, 1))


# ---------------------------------------------------------------------------
# Factory
# ---------------------------------------------------------------------------

_ITEM_MAP = {
    CsslWidgetType.BUTTON: ButtonItem,
    CsslWidgetType.LABEL: LabelItem,
    CsslWidgetType.INPUT_FIELD: InputFieldItem,
    CsslWidgetType.TEXT_AREA: TextAreaItem,
    CsslWidgetType.CHECK_BOX: CheckBoxItem,
    CsslWidgetType.RADIO_BUTTON: RadioButtonItem,
    CsslWidgetType.COMBO_BOX: ComboBoxItem,
    CsslWidgetType.SLIDER: SliderItem,
    CsslWidgetType.PROGRESS_BAR: ProgressBarItem,
    CsslWidgetType.LIST_BOX: ListBoxItem,
    CsslWidgetType.TABLE: TableItem,
    CsslWidgetType.SEPARATOR: SeparatorItem,
    CsslWidgetType.PICTURE: PictureItem,
    CsslWidgetType.TIME_PICKER: TimePickerItem,
    CsslWidgetType.DATE_PICKER: DatePickerItem,
    CsslWidgetType.TAB_CONTROL: TabControlItem,
    CsslWidgetType.SCROLL_PANEL: ScrollPanelItem,
    CsslWidgetType.MENU_BAR: MenuBarItem,
    CsslWidgetType.TOOLBAR: ToolbarItem,
}


def create_canvas_item(widget_model: WidgetModel,
                       parent_item: QGraphicsItem = None) -> CanvasWidgetItem:
    """Create the appropriate canvas item for a widget model."""
    item_class = _ITEM_MAP.get(widget_model.widget_type, CanvasWidgetItem)
    return item_class(widget_model, parent_item)



"""
CSSL GUI Builder - Widget hierarchy tree panel.
Displays the widget tree structure with context menu support.
"""

from PyQt6.QtWidgets import (QWidget, QVBoxLayout, QTreeWidget, QTreeWidgetItem,
                              QMenu, QInputDialog, QMessageBox, QAbstractItemView,
                              QHeaderView)
from PyQt6.QtCore import Qt, pyqtSignal
from PyQt6.QtGui import QFont, QColor, QAction

from .models import SceneModel, WidgetModel
from .constants import CsslWidgetType, WIDGET_REGISTRY
from .commands import CommandHistory, RemoveWidgetCommand, RenameWidgetCommand
from . import theme


class WidgetTreePanel(QWidget):
    """Bottom/side panel showing the widget hierarchy tree."""

    widget_selected = pyqtSignal(str)   # widget_id
    widget_deselected = pyqtSignal()

    def __init__(self, scene_model: SceneModel, command_history: CommandHistory,
                 parent: QWidget = None):
        super().__init__(parent)
        self._scene_model = scene_model
        self._command_history = command_history
        self._items_map: dict[str, QTreeWidgetItem] = {}
        self._updating = False

        self._setup_ui()
        self._scene_model.add_listener(self._on_model_change)

    def _setup_ui(self) -> None:
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0, 0, 0, 0)
        layout.setSpacing(0)

        self._tree = QTreeWidget()
        self._tree.setHeaderLabels(["Widget", "Type"])
        self._tree.setColumnCount(2)
        self._tree.setRootIsDecorated(True)
        self._tree.setSelectionMode(QAbstractItemView.SelectionMode.SingleSelection)
        self._tree.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
        self._tree.customContextMenuRequested.connect(self._show_context_menu)
        self._tree.currentItemChanged.connect(self._on_selection_changed)
        self._tree.setEditTriggers(QAbstractItemView.EditTrigger.NoEditTriggers)

        # Column sizing
        header = self._tree.header()
        header.setStretchLastSection(True)
        header.setSectionResizeMode(0, QHeaderView.ResizeMode.Stretch)
        header.setSectionResizeMode(1, QHeaderView.ResizeMode.ResizeToContents)

        layout.addWidget(self._tree)

    def rebuild(self) -> None:
        """Rebuild the entire tree from the scene model."""
        self._updating = True
        self._tree.clear()
        self._items_map.clear()

        # Add window as root if exists
        if self._scene_model.window:
            self._add_widget_item(self._scene_model.window, None)

        # Add orphan widgets (shouldn't normally happen, but safety)
        for widget in self._scene_model.get_all_widgets_ordered():
            if widget.id not in self._items_map:
                self._add_widget_item(widget, None)

        self._tree.expandAll()
        self._updating = False

    def _add_widget_item(self, widget: WidgetModel,
                          parent_item: QTreeWidgetItem | None) -> QTreeWidgetItem:
        """Add a widget and its children to the tree."""
        type_def = WIDGET_REGISTRY.get(widget.widget_type)
        icon = type_def.icon if type_def else ""
        display_name = f"{icon} {widget.name}" if widget.name else f"{icon} {widget.widget_type.value}"

        item = QTreeWidgetItem()
        item.setText(0, display_name)
        item.setText(1, widget.widget_type.value)
        item.setData(0, Qt.ItemDataRole.UserRole, widget.id)

        # Style based on widget type
        if widget.widget_type == CsslWidgetType.WIDGET:
            item.setFont(0, QFont("Segoe UI", 11, QFont.Weight.Bold))
        else:
            item.setFont(0, QFont("Segoe UI", 10))

        if parent_item:
            parent_item.addChild(item)
        else:
            self._tree.addTopLevelItem(item)

        self._items_map[widget.id] = item

        # Add children
        children = self._scene_model.get_children(widget.id)
        for child in sorted(children, key=lambda w: w.z_order):
            self._add_widget_item(child, item)

        return item

    def select_widget(self, widget_id: str) -> None:
        """Select a widget in the tree by ID."""
        item = self._items_map.get(widget_id)
        if item:
            self._updating = True
            self._tree.setCurrentItem(item)
            self._updating = False

    def deselect_all(self) -> None:
        self._updating = True
        self._tree.clearSelection()
        self._tree.setCurrentItem(None)
        self._updating = False

    # --- Signals ---

    def _on_selection_changed(self, current: QTreeWidgetItem,
                               previous: QTreeWidgetItem) -> None:
        if self._updating:
            return
        if current:
            widget_id = current.data(0, Qt.ItemDataRole.UserRole)
            if widget_id:
                self.widget_selected.emit(widget_id)
        else:
            self.widget_deselected.emit()

    # --- Context menu ---

    def _show_context_menu(self, pos) -> None:
        item = self._tree.itemAt(pos)
        if not item:
            return
        widget_id = item.data(0, Qt.ItemDataRole.UserRole)
        if not widget_id:
            return
        widget = self._scene_model.get_widget(widget_id)
        if not widget:
            return

        menu = QMenu(self)

        # Rename
        rename_action = QAction("Rename", self)
        rename_action.triggered.connect(lambda: self._rename_widget(widget_id))
        menu.addAction(rename_action)

        # Delete (not for window)
        if widget.widget_type != CsslWidgetType.WIDGET:
            menu.addSeparator()
            delete_action = QAction("Delete", self)
            delete_action.triggered.connect(lambda: self._delete_widget(widget_id))
            menu.addAction(delete_action)

        menu.exec(self._tree.mapToGlobal(pos))

    def _rename_widget(self, widget_id: str) -> None:
        widget = self._scene_model.get_widget(widget_id)
        if not widget:
            return

        new_name, ok = QInputDialog.getText(
            self, "Rename Widget", "New name:",
            text=widget.name
        )
        if ok and new_name.strip():
            new_name = new_name.strip()
            if self._scene_model.is_name_available(new_name, widget_id):
                cmd = RenameWidgetCommand(
                    self._scene_model, widget_id,
                    widget.name, new_name
                )
                self._command_history.execute(cmd)
            else:
                QMessageBox.warning(self, "Name Taken",
                                    f"The name '{new_name}' is already in use.")

    def _delete_widget(self, widget_id: str) -> None:
        widget = self._scene_model.get_widget(widget_id)
        if not widget or widget.widget_type == CsslWidgetType.WIDGET:
            return

        children = self._scene_model.get_children(widget_id)
        msg = f"Delete '{widget.name}'?"
        if children:
            msg += f"\nThis will also remove {len(children)} child widget(s)."

        reply = QMessageBox.question(
            self, "Delete Widget", msg,
            QMessageBox.StandardButton.Yes | QMessageBox.StandardButton.No
        )
        if reply == QMessageBox.StandardButton.Yes:
            cmd = RemoveWidgetCommand(self._scene_model, widget_id)
            self._command_history.execute(cmd)

    # --- Model change listener ---

    def _on_model_change(self, event: str, **kwargs) -> None:
        if event == "widget_added":
            self.rebuild()
        elif event == "widget_removed":
            self.rebuild()
        elif event == "widget_renamed":
            widget = kwargs.get("widget")
            if widget and widget.id in self._items_map:
                type_def = WIDGET_REGISTRY.get(widget.widget_type)
                icon = type_def.icon if type_def else ""
                self._items_map[widget.id].setText(
                    0, f"{icon} {widget.name}"
                )
        elif event == "scene_cleared":
            self._tree.clear()
            self._items_map.clear()



"""
CSSL Service Manager - Non-blocking toast notification popups.
"""
import tkinter as tk
from .. import theme


class Toast(tk.Toplevel):
    """A non-blocking notification popup that auto-dismisses."""

    _active_toasts = []
    _toast_offset = 0

    def __init__(self, parent, message: str, level: str = "info", duration: int = 3000):
        super().__init__(parent)
        self.overrideredirect(True)
        self.attributes("-topmost", True)

        colors = {
            "info": (theme.CARD, theme.TEXT, theme.BLUE),
            "success": (theme.CARD, theme.TEXT, theme.GREEN),
            "warning": (theme.CARD, theme.TEXT, theme.YELLOW),
            "error": (theme.CARD, theme.TEXT, theme.RED),
        }
        bg, fg, accent = colors.get(level, colors["info"])

        self.configure(bg=accent)

        inner = tk.Frame(self, bg=bg, padx=1, pady=1)
        inner.pack(fill="both", expand=True, padx=2, pady=2)

        # Icon
        icons = {"info": "\u2139", "success": "\u2713", "warning": "\u26A0", "error": "\u2717"}
        icon_text = icons.get(level, "\u2139")

        header = tk.Frame(inner, bg=bg)
        header.pack(fill="x", padx=8, pady=(6, 2))

        tk.Label(header, text=icon_text, font=(theme.FONT_FAMILY, 12), bg=bg, fg=accent
                 ).pack(side="left")
        tk.Label(header, text=level.upper(), font=theme.FONT_BOLD, bg=bg, fg=accent
                 ).pack(side="left", padx=(6, 0))

        # Close button
        close_btn = tk.Label(header, text="\u2715", font=theme.FONT_SMALL, bg=bg, fg=theme.TEXT_MUTED,
                             cursor="hand2")
        close_btn.pack(side="right")
        close_btn.bind("<Button-1>", lambda e: self._dismiss())

        # Message
        tk.Label(inner, text=message, font=theme.FONT, bg=bg, fg=fg,
                 wraplength=300, justify="left", anchor="w"
                 ).pack(fill="x", padx=8, pady=(0, 8))

        # Position: top-right of parent, stacked
        self.update_idletasks()
        Toast._active_toasts.append(self)
        idx = len(Toast._active_toasts) - 1

        pw = parent.winfo_width()
        px = parent.winfo_rootx()
        py = parent.winfo_rooty()
        tw = self.winfo_reqwidth()

        x = px + pw - tw - 16
        y = py + 48 + idx * (self.winfo_reqheight() + 8)
        self.geometry(f"+{x}+{y}")

        # Auto-dismiss
        self._dismiss_id = self.after(duration, self._dismiss)

    def _dismiss(self):
        try:
            self.after_cancel(self._dismiss_id)
        except Exception:
            pass
        if self in Toast._active_toasts:
            Toast._active_toasts.remove(self)
        try:
            self.destroy()
        except Exception:
            pass


def show_toast(parent, message: str, level: str = "info", duration: int = 3000):
    """Convenience function to show a toast notification."""
    return Toast(parent, message, level, duration)



"""
CSSL MessageBox Module - Simple message dialogs for CSSL
Version 4.9.7

Usage in CSSL:
    messagebox = include("cssl-gui.MessageBox");

    // Simple usage - show() waits by default
    msg = new messagebox(ok_btn=true);
    msg.setTitle("My Title");
    msg.setText("Hello World!");
    msg.show();  // Blocks until user clicks OK

    // Non-blocking usage
    msg.show(wait=false);  // Returns immediately
    // ... do other things ...
    msg.wait();  // Wait for dialog to close later

    // Convenience functions
    messagebox.showInfo("Title", "Message");
    messagebox.showWarning("Title", "Warning!");
    messagebox.showError("Title", "Error!");
    result = messagebox.askYesNo("Title", "Are you sure?");
    result = messagebox.askOkCancel("Title", "Continue?");
"""

import tkinter as tk
from tkinter import ttk
import threading
import os
import platform
from typing import Any, Callable, Dict, List, Optional

# Sound support
if platform.system() == 'Windows':
    import winsound
    SOUND_AVAILABLE = True
else:
    SOUND_AVAILABLE = False


def _strip_ansi(text: str) -> str:
    """Strip ANSI escape sequences (fmt:: formatting) from text"""
    import re
    ansi_escape = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
    return ansi_escape.sub('', str(text))


def _extract_ansi_color(text: str) -> Optional[str]:
    """Extract color from ANSI escape sequence and return tkinter color"""
    import re
    # ANSI color codes to tkinter colors
    ansi_to_color = {
        '30': '#000000',  # black
        '31': '#ff0000',  # red
        '32': '#00ff00',  # green
        '33': '#ffff00',  # yellow
        '34': '#0000ff',  # blue
        '35': '#ff00ff',  # magenta
        '36': '#00ffff',  # cyan
        '37': '#ffffff',  # white
        '90': '#808080',  # bright black (gray)
        '91': '#ff5555',  # bright red
        '92': '#55ff55',  # bright green
        '93': '#ffff55',  # bright yellow
        '94': '#5555ff',  # bright blue
        '95': '#ff55ff',  # bright magenta
        '96': '#55ffff',  # bright cyan
        '97': '#ffffff',  # bright white
    }
    # Match ANSI color code like \x1b[32m
    match = re.search(r'\x1B\[(\d+)m', str(text))
    if match:
        code = match.group(1)
        return ansi_to_color.get(code)
    return None


class MessageBoxButton:
    """A button in a MessageBox"""

    def __init__(self, parent: 'CsslMessageBox', text: str = "OK", is_default: bool = False):
        self._parent = parent
        self._text = text
        self._is_default = is_default
        self._callback: Optional[Callable] = None
        self._tk_button: Optional[tk.Button] = None
        self._visible = True
        self._color: Optional[str] = None

    def setText(self, text: str) -> 'MessageBoxButton':
        """Set button text (supports fmt:: colors)"""
        # Extract color before stripping ANSI codes
        color = _extract_ansi_color(text)
        clean_text = _strip_ansi(text)
        self._text = clean_text
        self._color = color
        if self._tk_button:
            self._tk_button.configure(text=clean_text)
            # Apply color if detected (need tk.Button, not ttk.Button for foreground)
        return self

    def setTextColor(self, color: str) -> 'MessageBoxButton':
        """Set button text color"""
        self._color = color
        return self

    def getText(self) -> str:
        """Get button text"""
        return self._text

    def onClick(self, callback: Callable) -> 'MessageBoxButton':
        """Set click callback"""
        self._callback = callback
        return self

    def on_click(self, callback: Callable) -> 'MessageBoxButton':
        """Alias for onClick"""
        return self.onClick(callback)

    def _handle_click(self) -> None:
        """Internal click handler"""
        # Set result to button text before calling callback
        self._parent._result = self._text

        if self._callback:
            try:
                self._callback()
            except Exception as e:
                print(f"[MessageBox] Button callback error: {e}")
        else:
            # Default: close the message box
            self._parent.close()

    def show(self) -> 'MessageBoxButton':
        """Show the button"""
        self._visible = True
        if self._tk_button:
            self._tk_button.pack(side=tk.LEFT, padx=5)
        return self

    def hide(self) -> 'MessageBoxButton':
        """Hide the button"""
        self._visible = False
        if self._tk_button:
            self._tk_button.pack_forget()
        return self

    def _create(self, parent_frame: tk.Frame) -> None:
        """Create the tkinter button"""
        # Use tk.Button (not ttk) to support foreground color
        self._tk_button = tk.Button(
            parent_frame,
            text=self._text,
            command=self._handle_click,
            padx=15,
            pady=5
        )
        # Apply color if set (from fmt:: formatting)
        if self._color:
            self._tk_button.configure(fg=self._color)
        if self._visible:
            self._tk_button.pack(side=tk.LEFT, padx=5)


class CsslMessageBox:
    """
    CSSL MessageBox - Simple message dialog

    Usage:
        msg = new CsslMessageBox(ok_btn=true, cancel_btn=true);
        msg.setTitle("Title");
        msg.setText("Message");
        msg.ok_btn.setText("Accept");
        msg.show();
    """

    def __init__(self, ok_btn: bool = True, cancel_btn: bool = False,
                 yes_btn: bool = False, no_btn: bool = False,
                 custom_btns: List[str] = None):
        self._title = "Message"
        self._text = ""
        self._width = 350
        self._height = 150
        self._running = False
        self._result: Optional[str] = None
        self._sound_path: Optional[str] = None
        self._bg_color = "#2d2d2d"
        self._fg_color = "#ffffff"
        self._window: Optional[tk.Tk] = None
        self._text_label: Optional[tk.Label] = None

        # Create buttons based on parameters
        self._buttons: Dict[str, MessageBoxButton] = {}

        if ok_btn:
            self._buttons['ok_btn'] = MessageBoxButton(self, "OK", is_default=True)
            self.ok_btn = self._buttons['ok_btn']

        if cancel_btn:
            self._buttons['cancel_btn'] = MessageBoxButton(self, "Cancel")
            self.cancel_btn = self._buttons['cancel_btn']

        if yes_btn:
            self._buttons['yes_btn'] = MessageBoxButton(self, "Yes", is_default=True)
            self.yes_btn = self._buttons['yes_btn']

        if no_btn:
            self._buttons['no_btn'] = MessageBoxButton(self, "No")
            self.no_btn = self._buttons['no_btn']

        if custom_btns:
            for i, btn_text in enumerate(custom_btns):
                btn_name = f'btn_{i}'
                self._buttons[btn_name] = MessageBoxButton(self, btn_text)
                setattr(self, btn_name, self._buttons[btn_name])

    def setTitle(self, title: str) -> 'CsslMessageBox':
        """Set the message box title"""
        self._title = title
        if self._window:
            self._window.title(title)
        return self

    def setText(self, text: str) -> 'CsslMessageBox':
        """Set the message text (supports fmt:: formatting)"""
        # Strip ANSI/fmt codes for display (tkinter doesn't support them)
        clean_text = self._strip_formatting(text)
        self._text = clean_text
        if self._text_label:
            self._text_label.configure(text=clean_text)
        return self

    def _strip_formatting(self, text: str) -> str:
        """Strip fmt:: formatting codes (ANSI escape sequences)"""
        import re
        # Remove ANSI escape sequences
        ansi_escape = re.compile(r'\x1B(?:[@-Z\\-_]|\[[0-?]*[ -/]*[@-~])')
        return ansi_escape.sub('', str(text))

    def setSize(self, width: int, height: int) -> 'CsslMessageBox':
        """Set the message box size"""
        self._width = width
        self._height = height
        if self._window:
            self._window.geometry(f"{width}x{height}")
        return self

    def setBackgroundColor(self, color: str) -> 'CsslMessageBox':
        """Set background color"""
        self._bg_color = color
        return self

    def setTextColor(self, color: str) -> 'CsslMessageBox':
        """Set text color"""
        self._fg_color = color
        return self

    def playSound(self, path: str) -> 'CsslMessageBox':
        """Set sound to play when message box appears"""
        self._sound_path = path
        return self

    def _play_sound(self) -> None:
        """Play the configured sound"""
        if not self._sound_path or not SOUND_AVAILABLE:
            return

        if not os.path.exists(self._sound_path):
            return

        def _do_play():
            try:
                if self._sound_path.lower().endswith('.wav'):
                    winsound.PlaySound(self._sound_path, winsound.SND_FILENAME | winsound.SND_ASYNC)
            except:
                pass

        threading.Thread(target=_do_play, daemon=True).start()

    def addButton(self, text: str, callback: Callable = None) -> MessageBoxButton:
        """Add a custom button"""
        btn_name = f'btn_{len(self._buttons)}'
        btn = MessageBoxButton(self, text)
        if callback:
            btn.onClick(callback)
        self._buttons[btn_name] = btn
        setattr(self, btn_name, btn)
        return btn

    def show(self, wait: bool = True) -> 'CsslMessageBox':
        """Show the message box

        Args:
            wait: If True (default), blocks until the dialog is closed.
                  If False, returns immediately (use wait() later).
        """
        self._create_window()
        self._running = True

        # Play sound if configured
        if self._sound_path:
            self._play_sound()

        # By default, wait for user to close the dialog
        if wait:
            self.wait()

        return self

    def _create_window(self) -> None:
        """Create the tkinter window"""
        # Create as a standalone Tk window (simpler, more reliable)
        self._window = tk.Tk()
        self._window.title(self._title)
        self._window.configure(bg=self._bg_color)
        self._window.resizable(False, False)
        self._window.protocol("WM_DELETE_WINDOW", self._on_close)

        # Center on screen
        screen_width = self._window.winfo_screenwidth()
        screen_height = self._window.winfo_screenheight()
        x = (screen_width // 2) - (self._width // 2)
        y = (screen_height // 2) - (self._height // 2)
        self._window.geometry(f"{self._width}x{self._height}+{x}+{y}")

        # Make it stay on top
        self._window.attributes('-topmost', True)
        self._window.focus_force()

        # Create content frame
        content_frame = tk.Frame(self._window, bg=self._bg_color)
        content_frame.pack(expand=True, fill=tk.BOTH, padx=20, pady=15)

        # Text label
        self._text_label = tk.Label(
            content_frame,
            text=self._text,
            bg=self._bg_color,
            fg=self._fg_color,
            font=("Arial", 11),
            wraplength=self._width - 50,
            justify=tk.LEFT
        )
        self._text_label.pack(expand=True, fill=tk.BOTH)

        # Button frame
        button_frame = tk.Frame(self._window, bg=self._bg_color)
        button_frame.pack(side=tk.BOTTOM, pady=15)

        # Create buttons
        for btn in self._buttons.values():
            btn._create(button_frame)

    def _on_close(self) -> None:
        """Handle window close"""
        self._running = False
        if self._result is None:
            self._result = "closed"
        if self._window:
            try:
                self._window.grab_release()
                self._window.destroy()
            except tk.TclError:
                pass
            self._window = None

    def close(self) -> None:
        """Close the message box"""
        self._on_close()

    def hide(self) -> 'CsslMessageBox':
        """Hide the message box (can be shown again)"""
        if self._window:
            self._window.withdraw()
        return self

    def isRunning(self) -> bool:
        """Check if the message box is still open"""
        return self._running

    def getResult(self) -> Optional[str]:
        """Get the result (which button was clicked)"""
        return self._result

    def wait(self, timeout_ms: int = None) -> Optional[str]:
        """Wait for the message box to close and return result

        Args:
            timeout_ms: If provided, only wait this many milliseconds then return.
                        If None (default), blocks until dialog closes.
        """
        import time

        if timeout_ms is not None:
            # Non-blocking wait with timeout - just process events once
            if self._window and self._running:
                try:
                    self._window.update()
                    if self._window:  # Check again after update
                        self._window.update_idletasks()
                except tk.TclError:
                    self._running = False
        else:
            # Blocking wait - manually pump events until window closes
            while self._running and self._window:
                try:
                    self._window.update()
                    if not self._window or not self._running:
                        break
                    self._window.update_idletasks()
                    time.sleep(0.01)  # Prevent CPU spinning
                except tk.TclError:
                    self._running = False
                    break

        return self._result

    def mainloop(self) -> None:
        """Run the message box event loop (alias for wait())"""
        self.wait()

    def update(self) -> None:
        """Process pending events"""
        if self._window:
            try:
                self._window.update()
            except tk.TclError:
                self._running = False


# Convenience functions
def showInfo(title: str, message: str) -> None:
    """Show an info message box"""
    msg = CsslMessageBox(ok_btn=True)
    msg.setTitle(title)
    msg.setText(message)
    msg.show()  # wait=True by default


def showWarning(title: str, message: str) -> None:
    """Show a warning message box"""
    msg = CsslMessageBox(ok_btn=True)
    msg.setTitle(title)
    msg.setText(message)
    msg.setBackgroundColor("#4a3c00")
    msg.show()  # wait=True by default


def showError(title: str, message: str) -> None:
    """Show an error message box"""
    msg = CsslMessageBox(ok_btn=True)
    msg.setTitle(title)
    msg.setText(message)
    msg.setBackgroundColor("#4a0000")
    msg.show()  # wait=True by default


def askYesNo(title: str, message: str) -> bool:
    """Show a yes/no question and return True for Yes"""
    result = [None]

    def on_yes():
        result[0] = True
        msg.close()

    def on_no():
        result[0] = False
        msg.close()

    msg = CsslMessageBox(yes_btn=True, no_btn=True)
    msg.setTitle(title)
    msg.setText(message)
    msg.yes_btn.onClick(on_yes)
    msg.no_btn.onClick(on_no)
    msg.show()  # wait=True by default

    return result[0] if result[0] is not None else False


def askOkCancel(title: str, message: str) -> bool:
    """Show an OK/Cancel question and return True for OK"""
    result = [None]

    def on_ok():
        result[0] = True
        msg.close()

    def on_cancel():
        result[0] = False
        msg.close()

    msg = CsslMessageBox(ok_btn=True, cancel_btn=True)
    msg.setTitle(title)
    msg.setText(message)
    msg.ok_btn.onClick(on_ok)
    msg.cancel_btn.onClick(on_cancel)
    msg.show()  # wait=True by default

    return result[0] if result[0] is not None else False


class CsslMessageBoxModule:
    """Module returned by include("cssl-gui.MessageBox")"""

    # The main class
    MessageBox = CsslMessageBox

    # Convenience functions
    showInfo = staticmethod(showInfo)
    showWarning = staticmethod(showWarning)
    showError = staticmethod(showError)
    askYesNo = staticmethod(askYesNo)
    askOkCancel = staticmethod(askOkCancel)

    def __new__(cls, *args, **kwargs):
        """Allow: msg = new messagebox(ok_btn=true)"""
        return CsslMessageBox(*args, **kwargs)

    def __init__(self, *args, **kwargs):
        pass


# Global module instance
_messagebox_module = None


def get_messagebox_module():
    """Get the MessageBox module"""
    global _messagebox_module
    if _messagebox_module is None:
        _messagebox_module = CsslMessageBoxModule
    return _messagebox_module


__all__ = [
    'CsslMessageBox',
    'MessageBoxButton',
    'CsslMessageBoxModule',
    'get_messagebox_module',
    'showInfo',
    'showWarning',
    'showError',
    'askYesNo',
    'askOkCancel',
]
